using UnityEngine;
using System.Collections.Generic;

namespace Mod
{
    public class EMP1 : MonoBehaviour
    {

        public bool Activated = true;

        private Vector2 barrelDirection = Vector2.up;

        private Transform gripper;
        private RaycastHit2D[] buffer = new RaycastHit2D[16];

        private PhysicalBehaviour physicalBehaviour;
        private SpriteRenderer spriteRenderer;
        private SpriteRenderer beamRenderer;
        private GameObject beam;
        private AudioSource audio;
        private AudioSource Audio;

        private void Start()
        {
            beam = new GameObject("components", typeof(Optout));
            beam.transform.localScale = transform.localScale;
            beam.transform.SetParent(transform);
            beam.transform.eulerAngles = beam.transform.eulerAngles;
            beam.transform.localPosition = new Vector3(0f, 0f) * Mod.PixelSize;
            beamRenderer = beam.AddComponent<SpriteRenderer>();
            beamRenderer.sprite = Cyborg.EnergySwordBlade2;
            beamRenderer.sharedMaterial = ModAPI.FindMaterial("VeryBright");
            //beamRenderer.color = new Color(0.5f, 0.5f, 0.5f);
            beamRenderer.sortingLayerName = "Default";
            var light = ModAPI.CreateLight(transform, Color.cyan, 6, 1);

            audio = beam.GetOrAddComponent<AudioSource>();
            audio.spatialBlend = 0.1f;
            audio.minDistance = 1;
            audio.maxDistance = 3;
            audio.playOnAwake = false;
            audio.loop = true;
            audio.clip = Cyborg.PlasmaPistolLoop;

            SetActivated(Activated);
        }

        private void Awake()
        {
            physicalBehaviour = GetComponent<PhysicalBehaviour>();
            spriteRenderer = GetComponent<SpriteRenderer>();
            SetActivated(!Activated);
        }

        private void FixedUpdate()
        {
            if (Activated)
                CastDamageLine(transform.position, transform.position + transform.TransformDirection(barrelDirection) * 70 * Mod.PixelSize * transform.localScale.normalized.y);
        }

        private void LateUpdate()
        {
            beamRenderer.color = Color.Lerp(new Color(0.15f, 0.15f, 0.15f), new Color(0.15f, 0.15f, 0.15f), UnityEngine.Random.value);
        }

        private void SetActivated(bool a)
        {
            Activated = a;

            beam.SetActive(a);

            if (a)
            {
                //audio.Play();
                audio.PlayOneShot(Cyborg.EnergySwordIgnite);
            }
            else
            {
                //audio.Stop();
                audio.PlayOneShot(Cyborg.EnergySwordIgnite);
            }
        }

        private void CastDamageLine(Vector2 start, Vector2 end)
        {
            if (physicalBehaviour.beingHeldByGripper && gripper == null)
            {
                var near = Physics2D.OverlapPointAll(transform.position);
                foreach (var item in near)
                {
                    if (item.GetComponent<GripBehaviour>())
                    {
                        gripper = item.transform.root;
                        break;
                    }
                }
            }
            else if (!physicalBehaviour.beingHeldByGripper)
                gripper = null;

            var rayHits = Physics2D.LinecastNonAlloc(start, end, buffer);
            List<GameObject> hitGameObjects = new List<GameObject>();
            for (int i = 0; i < rayHits; i++)
            {
                var hit = buffer[i];
                if (hit.transform.root == transform.root) continue;
                if (gripper && hit.transform.root == gripper) continue;
                if (Random.value > 0.1f)
                {
            var rb = hit.transform.GetComponent<Rigidbody2D>();
            if(rb.velocity.magnitude >= -100f)
            {
               hit.transform.GetComponent<Rigidbody2D>().velocity = rb.velocity * 2;
            }
                    var effect = ModAPI.CreateParticleEffect("Vapor", hit.point);
                    var main = effect.GetComponent<ParticleSystem>().main;main.startColor = new Color(0f, 150f, 300f, 300f);
                    effect.AddComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("timeball.png");
                    var effect2 = ModAPI.CreateParticleEffect("Vapor", hit.point);
                    var main1 = effect2.GetComponent<ParticleSystem>().main;main1.startColor = new Color(0f, 150f, 300f, 300f);
                    effect2.AddComponent<SpriteRenderer>().sprite = ModAPI.LoadSprite("timeball.png");
              }  
           }
        }   
      }  
   }
